package com.trendify.inventory.service;


import com.trendify.inventory.dto.InventoryDto;
import com.trendify.inventory.entity.Inventory;
import com.trendify.inventory.repository.InventoryRepository;
import com.trendify.inventory.utils.AppUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;
import reactor.core.publisher.Mono;

import java.util.logging.Logger;

@Service
public class InventoryService {

    @Autowired
    private InventoryRepository repository;

    public Mono<InventoryDto> saveProduct(Mono<InventoryDto> inventoryDtoMono) {
        return inventoryDtoMono
                .flatMap(dto -> repository.findByProductId(dto.getProductId())
                        .flatMap(existingEntry -> {
                            System.out.println("EXISTS !! Duplicate Records.");
                            return Mono.<InventoryDto>error(new ResponseStatusException(HttpStatus.CONFLICT, "Product already exists"));
                        })
                        .switchIfEmpty(Mono.defer(() -> {
                            return Mono.just(dto)
                                    .map(AppUtils::dtoToEntity)
                                    .flatMap(repository::save)
                                    .map(AppUtils::entityToDto);
                        })));
    }

    public Mono<InventoryDto> updateProduct(Mono<InventoryDto> inventoryDtoMono, String id) {
        return repository.findByProductId(id)
                .flatMap(existingProduct -> inventoryDtoMono.flatMap(dto -> {
                    if (dto.getProductId().equals(id)) {
                        // Updating the same product, proceed.
                        Inventory updatedInventory = AppUtils.dtoToEntity(dto);
                        updatedInventory.setId(existingProduct.getId()); // Set the _id.
                        updatedInventory.setProductId(id);
                        return repository.save(updatedInventory).map(AppUtils::entityToDto);
                    } else {
                        // Check if the new productId already exists.
                        return repository.findByProductId(dto.getProductId())
                                .flatMap(conflictingProduct -> {
                                    // Another product with the new productId exists.
                                    return Mono.<InventoryDto>error(new ResponseStatusException(HttpStatus.CONFLICT, "Product with the given productId already exists"));
                                })
                                .switchIfEmpty(Mono.defer(() -> {
                                    // No conflict, proceed with the update.
                                    Inventory updatedInventory = AppUtils.dtoToEntity(dto);
                                    updatedInventory.setId(existingProduct.getId()); // Set the _id.
                                    updatedInventory.setProductId(id);
                                    return repository.save(updatedInventory).map(AppUtils::entityToDto);
                                }));
                    }
                }));
    }



    public Mono<Integer> getQuantity(String id){
        return repository.findByProductId(id).map(AppUtils::entityToDto).map(dto -> dto.getQuantity());
    }
}
